@@ -17,44 +17,22 @@
 
 package org.apache.eventmesh.common.protocol.tcp;
 
-public class Package {
+import org.apache.eventmesh.common.protocol.ProtocolTransportObject;
+
+import lombok.AllArgsConstructor;
+import lombok.Data;
+import lombok.NoArgsConstructor;
+
+@Data
+@NoArgsConstructor
+@AllArgsConstructor
+public class Package implements ProtocolTransportObject {
 
     private Header header;
     private Object body;
 
-    public Package() {
-    }
-
     public Package(Header header) {
         this.header = header;
     }
 
-    public Package(Header header, Object body) {
-        this.header = header;
-        this.body = body;
-    }
-
-    public Header getHeader() {
-        return header;
-    }
-
-    public void setHeader(Header header) {
-        this.header = header;
-    }
-
-    public Object getBody() {
-        return body;
-    }
-
-    public void setBody(Object body) {
-        this.body = body;
-    }
-
-    @Override
-    public String toString() {
-        return "Package{" +
-                "header=" + header +
-                ", body=" + body +
-                '}';
-    }
 }
