@@ -19,7 +19,7 @@ package org.apache.eventmesh.runtime.core.protocol.http.processor.inf;
 
 import io.netty.channel.ChannelHandlerContext;
 
-import org.apache.eventmesh.common.command.HttpCommand;
+import org.apache.eventmesh.common.protocol.http.HttpCommand;
 import org.apache.eventmesh.runtime.core.protocol.http.async.AsyncContext;
 
 public interface HttpRequestProcessor {
