@@ -17,21 +17,18 @@
 
 package org.apache.eventmesh.runtime.core.protocol.tcp.client.session.send;
 
-import java.util.concurrent.Semaphore;
-import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicLong;
-
-import io.openmessaging.api.Message;
-import io.openmessaging.api.SendCallback;
-
+import io.cloudevents.core.builder.CloudEventBuilder;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.commons.lang3.time.DateFormatUtils;
-import org.apache.eventmesh.api.RRCallback;
+import org.apache.eventmesh.api.RequestReplyCallback;
+import org.apache.eventmesh.api.SendCallback;
 import org.apache.eventmesh.common.Constants;
 import org.apache.eventmesh.common.protocol.tcp.Command;
 import org.apache.eventmesh.common.protocol.tcp.Header;
 import org.apache.eventmesh.common.protocol.tcp.OPStatus;
 import org.apache.eventmesh.common.protocol.tcp.Package;
+import org.apache.eventmesh.protocol.api.ProtocolAdaptor;
+import org.apache.eventmesh.protocol.api.ProtocolPluginFactory;
 import org.apache.eventmesh.runtime.constants.EventMeshConstants;
 import org.apache.eventmesh.runtime.core.protocol.tcp.client.session.Session;
 import org.apache.eventmesh.runtime.util.EventMeshUtil;
@@ -39,6 +36,13 @@ import org.apache.eventmesh.runtime.util.Utils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.util.Objects;
+import java.util.concurrent.Semaphore;
+import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicLong;
+
+import io.cloudevents.CloudEvent;
+
 public class SessionSender {
 
     private final Logger messageLogger = LoggerFactory.getLogger("message");
@@ -73,22 +77,30 @@ public class SessionSender {
         this.upstreamBuff = new Semaphore(session.getEventMeshTCPConfiguration().eventMeshTcpSessionUpstreamBufferSize);
     }
 
-    public EventMeshTcpSendResult send(Header header, Message msg, SendCallback sendCallback, long startTime, long taskExecuteTime) {
+    public EventMeshTcpSendResult send(Header header, CloudEvent event, SendCallback sendCallback, long startTime, long taskExecuteTime) {
         try {
             if (upstreamBuff.tryAcquire(TRY_PERMIT_TIME_OUT, TimeUnit.MILLISECONDS)) {
                 upMsgs.incrementAndGet();
                 UpStreamMsgContext upStreamMsgContext = null;
-                Command cmd = header.getCommand();
+                Command cmd = header.getCmd();
+                long ttl = EventMeshConstants.DEFAULT_TIMEOUT_IN_MILLISECONDS;
                 if (Command.REQUEST_TO_SERVER == cmd) {
-                    long ttl = msg.getSystemProperties(EventMeshConstants.PROPERTY_MESSAGE_TTL) != null ? Long.parseLong(msg.getSystemProperties(EventMeshConstants.PROPERTY_MESSAGE_TTL)) : EventMeshConstants.DEFAULT_TIMEOUT_IN_MILLISECONDS;
-                    upStreamMsgContext = new UpStreamMsgContext(header.getSeq(), session, msg);
-                    session.getClientGroupWrapper().get().request(upStreamMsgContext, sendCallback, initSyncRRCallback(header, startTime, taskExecuteTime), ttl);
+                    if (event.getExtension(EventMeshConstants.PROPERTY_MESSAGE_TTL) != null) {
+                       ttl = Long.parseLong((String) Objects.requireNonNull(event.getExtension(EventMeshConstants.PROPERTY_MESSAGE_TTL)));
+                    }
+//                    long ttl = msg.getSystemProperties(EventMeshConstants.PROPERTY_MESSAGE_TTL) != null ? Long.parseLong(msg.getSystemProperties(EventMeshConstants.PROPERTY_MESSAGE_TTL)) : EventMeshConstants.DEFAULT_TIMEOUT_IN_MILLISECONDS;
+                    upStreamMsgContext = new UpStreamMsgContext(session, event, header, startTime, taskExecuteTime);
+                    session.getClientGroupWrapper().get().request(upStreamMsgContext, initSyncRRCallback(header, startTime, taskExecuteTime), ttl);
+                    upstreamBuff.release();
                 } else if (Command.RESPONSE_TO_SERVER == cmd) {
-                    String cluster = msg.getUserProperties(EventMeshConstants.PROPERTY_MESSAGE_CLUSTER);
+                    String cluster = (String)event.getExtension(EventMeshConstants.PROPERTY_MESSAGE_CLUSTER);
+//                    String cluster = msg.getUserProperties(EventMeshConstants.PROPERTY_MESSAGE_CLUSTER);
                     if (!StringUtils.isEmpty(cluster)) {
                         String replyTopic = EventMeshConstants.RR_REPLY_TOPIC;
                         replyTopic = cluster + "-" + replyTopic;
-                        msg.getSystemProperties().put(Constants.PROPERTY_MESSAGE_DESTINATION, replyTopic);
+                        event = CloudEventBuilder.from(event).withSubject(replyTopic).build();
+//                        msg.getSystemProperties().put(Constants.PROPERTY_MESSAGE_DESTINATION, replyTopic);
+//                        event(replyTopic);
                     }
 
 //                    //for rocketmq support
@@ -96,11 +108,11 @@ public class SessionSender {
 //                    MessageAccessor.putProperty(msg, MessageConst.PROPERTY_CORRELATION_ID, msg.getProperty(DeFiBusConstant.PROPERTY_RR_REQUEST_ID));
 //                    MessageAccessor.putProperty(msg, MessageConst.PROPERTY_MESSAGE_REPLY_TO_CLIENT, msg.getProperty(DeFiBusConstant.PROPERTY_MESSAGE_REPLY_TO));
 
-                    upStreamMsgContext = new UpStreamMsgContext(header.getSeq(), session, msg);
+                    upStreamMsgContext = new UpStreamMsgContext(session, event, header, startTime, taskExecuteTime);
                     session.getClientGroupWrapper().get().reply(upStreamMsgContext);
                     upstreamBuff.release();
                 } else {
-                    upStreamMsgContext = new UpStreamMsgContext(header.getSeq(), session, msg);
+                    upStreamMsgContext = new UpStreamMsgContext(session, event, header, startTime, taskExecuteTime);
                     session.getClientGroupWrapper().get().send(upStreamMsgContext, sendCallback);
                 }
 
@@ -120,36 +132,46 @@ public class SessionSender {
         return new EventMeshTcpSendResult(header.getSeq(), EventMeshTcpSendStatus.SUCCESS, EventMeshTcpSendStatus.SUCCESS.name());
     }
 
-    private RRCallback initSyncRRCallback(Header header, long startTime, long taskExecuteTime) {
-        return new RRCallback() {
+    private RequestReplyCallback initSyncRRCallback(Header header, long startTime, long taskExecuteTime) {
+        return new RequestReplyCallback() {
             @Override
-            public void onSuccess(Message msg) {
+            public void onSuccess(CloudEvent event) {
                 String seq = header.getSeq();
-                //TODO 此处如何赋值
+                // TODO: How to assign values here
 //                if (msg instanceof MessageExt) {
 //                    msg.putUserProperty(EventMeshConstants.BORN_TIMESTAMP, String.valueOf(((MessageExt) msg)
 //                            .getBornTimestamp()));
 //                    msg.putUserProperty(EventMeshConstants.STORE_TIMESTAMP, String.valueOf(((MessageExt) msg)
 //                            .getStoreTimestamp()));
 //                }
-
-                msg.getSystemProperties().put(EventMeshConstants.RSP_MQ2EVENTMESH_TIMESTAMP, String.valueOf(System.currentTimeMillis()));
-                msg.getSystemProperties().put(EventMeshConstants.RSP_RECEIVE_EVENTMESH_IP, session.getEventMeshTCPConfiguration().eventMeshServerIp);
+                event = CloudEventBuilder.from(event)
+                        .withExtension(EventMeshConstants.RSP_MQ2EVENTMESH_TIMESTAMP, String.valueOf(System.currentTimeMillis()))
+                        .withExtension(EventMeshConstants.RSP_RECEIVE_EVENTMESH_IP, session.getEventMeshTCPConfiguration().eventMeshServerIp)
+                        .build();
                 session.getClientGroupWrapper().get().getEventMeshTcpMonitor().getMq2EventMeshMsgNum().incrementAndGet();
 
                 Command cmd;
-                if (header.getCommand().equals(Command.REQUEST_TO_SERVER)) {
+                if (header.getCmd().equals(Command.REQUEST_TO_SERVER)) {
                     cmd = Command.RESPONSE_TO_CLIENT;
                 } else {
-                    messageLogger.error("invalid message|messageHeader={}|msg={}", header, msg);
+                    messageLogger.error("invalid message|messageHeader={}|event={}", header, event);
                     return;
                 }
+                event = CloudEventBuilder.from(event)
+                    .withExtension(EventMeshConstants.RSP_EVENTMESH2C_TIMESTAMP, String.valueOf(System.currentTimeMillis()))
+                    .build();
+                String protocolType = Objects.requireNonNull(event.getExtension(Constants.PROTOCOL_TYPE)).toString();
+
+                ProtocolAdaptor protocolAdaptor = ProtocolPluginFactory.getProtocolAdaptor(protocolType);
+
                 Package pkg = new Package();
-                pkg.setHeader(new Header(cmd, OPStatus.SUCCESS.getCode(), null, seq));
-                msg.getSystemProperties().put(EventMeshConstants.RSP_EVENTMESH2C_TIMESTAMP, String.valueOf(System.currentTimeMillis()));
+
+//                msg.getSystemProperties().put(EventMeshConstants.RSP_EVENTMESH2C_TIMESTAMP, String.valueOf(System.currentTimeMillis()));
                 try {
-                    pkg.setBody(EventMeshUtil.encodeMessage(msg));
+//                    pkg.setBody(EventMeshUtil.encodeMessage(msg));
+                    pkg = (Package) protocolAdaptor.fromCloudEvent(event);
                     pkg.setHeader(new Header(cmd, OPStatus.SUCCESS.getCode(), null, seq));
+                    pkg.getHeader().putProperty(Constants.PROTOCOL_TYPE, protocolType);
                 } catch (Exception e) {
                     pkg.setHeader(new Header(cmd, OPStatus.FAIL.getCode(), null, seq));
                 } finally {
